# example config data for Dist::Maker
{
    user => {
        name  => '<< your name here >>',
        email => '<< your email address here >>',
    },
    verbose   => 3,
    namespace => ['My::Framework::Dist'],
    module    => ['Text::Xslate::Bridge::TT2LIke'],
};
